/*
 * Copyright 2002 (C) Michal Zawalich <michuz@pld.org.pl>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <fcntl.h>
#include <stdlib.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <pthread.h>

#include "progress.h"
#include "callbacks.h"
#include "cddb_handler.h"
#include "cdrom_handler.h"
#include "error.h"
#include "interface.h"
#include "settings.h"

/* I don't want use directly GtkWidget *appbar */
gchar *barinfo;

/********************************************************************/
void on_exit_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	gtk_main_quit();
}

/********************************************************************/
void on_preferences_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	create_settings_window();
}

/********************************************************************/
void on_about_activate(GtkMenuItem * menuitem, gpointer user_data)
{

}

/********************************************************************/
void on_scancd_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	GtkTreeIter iter;
	GtkWidget *tracks_list_model = user_data;
	struct track_info *info;
	gint i;

	/* clear list */
	gtk_list_store_clear(GTK_LIST_STORE(tracks_list_model));

	if (cdrom_scan() == GR_SUCCESS) {
		/* appbar info */
		gnome_appbar_push(GNOME_APPBAR(appbar), barinfo);
		g_free(barinfo);

		for (i = 1; i <= g_slist_length(tracks_list); i++) {
			info = (struct track_info *)
			    g_slist_nth_data(tracks_list, i - 1);

			/* merge data to list */
			gtk_list_store_append(GTK_LIST_STORE
					      (tracks_list_model), &iter);
			gtk_list_store_set(GTK_LIST_STORE
					   (tracks_list_model), &iter,
					   COLUMN_SELECTED, info->selected,
					   COLUMN_TRACK,
					   g_strdup_printf("%02d",
							   info->track),
					   COLUMN_TIME, info->time,
					   COLUMN_ARTIST, info->artist,
					   COLUMN_TITLE, info->title,
					   COLUMN_ALBUM, info->album,
					   COLUMN_YEAR, info->year,
					   COLUMN_GENRE, info->genre,
					   COLUMN_COMMENT, info->comment,
					   COLUMN_EDITABLE, TRUE, -1);
		}
	} else {
		gr_error();
		return;
	}
}

/********************************************************************/
void on_cell_edited_activate(GtkCellRendererText * cell,
			gchar * path_string, gchar * new_text,
			gpointer data)
{
	GtkTreeModel *model = (GtkTreeModel *) data;
	GtkTreePath *path = gtk_tree_path_new_from_string(path_string);
	GtkTreeIter iter;
	gint *column;
	gchar *old_text = NULL;
	gchar *track;
	struct track_info *info;

	column = g_object_get_data(G_OBJECT(cell), "column");
	gtk_tree_model_get_iter(model, &iter, path);

	/* get the track number and find track data */
	gtk_tree_model_get(model, &iter, COLUMN_TRACK, &track, -1);
	info = g_slist_nth_data(tracks_list, atoi(track) - 1);

	switch ((gint) column) {
	case COLUMN_ARTIST:
		gtk_tree_model_get(model, &iter, column, &old_text, -1);
		gtk_list_store_set(GTK_LIST_STORE(model), &iter, column, new_text, -1);
		g_free(old_text);
		g_free(info->artist);
		info->artist = g_strdup(new_text);
		break;
	case COLUMN_TITLE:
		gtk_tree_model_get(model, &iter, column, &old_text, -1);
		gtk_list_store_set(GTK_LIST_STORE(model), &iter, column, new_text, -1);
		g_free(old_text);
		g_free(info->title);
		info->title = g_strdup(new_text);
		break;
	case COLUMN_ALBUM:
		gtk_tree_model_get(model, &iter, column, &old_text, -1);
		gtk_list_store_set(GTK_LIST_STORE(model), &iter, column, new_text, -1);
		g_free(old_text);
		g_free(info->album);
		info->album = g_strdup(new_text);
		break;
	case COLUMN_YEAR:
		gtk_tree_model_get(model, &iter, column, &old_text, -1);
		gtk_list_store_set(GTK_LIST_STORE(model), &iter, column, new_text, -1);
		g_free(old_text);
		g_free(info->year);
		info->year = g_strdup(new_text);
		break;
	case COLUMN_GENRE:
		gtk_tree_model_get(model, &iter, column, &old_text, -1);
		gtk_list_store_set(GTK_LIST_STORE(model), &iter, column, new_text, -1);
		g_free(old_text);
		g_free(info->genre);
		info->genre = g_strdup(new_text);
		break;
	case COLUMN_COMMENT:
		gtk_tree_model_get(model, &iter, column, &old_text, -1);
		gtk_list_store_set(GTK_LIST_STORE(model), &iter, column, new_text, -1);
		g_free(old_text);
		g_free(info->comment);
		info->comment = g_strdup(new_text);
		break;
	}
	gtk_tree_path_free(path);
}

/********************************************************************/
void
on_cell_toggled_activate(GtkCellRendererToggle * cell,
			 gchar * path_str, gpointer data)
{
	GtkTreeModel *model = (GtkTreeModel *) data;
	GtkTreeIter iter;
	GtkTreePath *path = gtk_tree_path_new_from_string(path_str);
	struct track_info *info;
	gboolean selected;
	gchar *tmp;

	/* get toggled iter */
	gtk_tree_model_get_iter(model, &iter, path);
	gtk_tree_model_get(model, &iter, COLUMN_SELECTED, &selected,
			   COLUMN_TRACK, &tmp, -1);
	info = (struct track_info *) g_slist_nth_data(tracks_list,
						      atoi(tmp) - 1);

	/* do something with the value */
	selected ^= 1;
	info->selected ^= 1;

	/* set new value */
	gtk_list_store_set(GTK_LIST_STORE(model), &iter, COLUMN_SELECTED,
			   selected, -1);

	/* clean up */
	gtk_tree_path_free(path);
}

/********************************************************************/
void on_cddb_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	GtkTreeIter iter;
	GtkWidget *tracks_list_model = user_data;
	struct track_info *info;
	gint i;

	g_print("quering fir disc %02x...\n", cddb_discid());
	if (cddb_do_query() == TRUE) {
		switch (state.cddb_match) {
		case CDDB_MATCH_INEXACT:
		case CDDB_MATCH_EXACT:
			g_print("match for \"%s - %s\"\n",
				state.cddb_list_artist,
				state.cddb_list_title);
			g_print("downloading data...\n");
			cddb_read();
			g_print("done\n");
			/* clear list */
			gtk_list_store_clear(GTK_LIST_STORE
					     (tracks_list_model));
			for (i = 1; i <= g_slist_length(tracks_list); i++) {
				info =
				    g_slist_nth_data(tracks_list, i - 1);
				/* merge data to list */
				gtk_list_store_append(GTK_LIST_STORE
						      (tracks_list_model),
						      &iter);
				gtk_list_store_set(GTK_LIST_STORE
						   (tracks_list_model),
						   &iter, COLUMN_SELECTED,
						   info->selected,
						   COLUMN_TRACK,
						   g_strdup_printf("%02d",
								   info->
								   track),
						   COLUMN_TIME, info->time,
						   COLUMN_ARTIST,
						   info->artist,
						   COLUMN_TITLE,
						   info->title,
						   COLUMN_ALBUM,
						   info->album,
						   COLUMN_YEAR, info->year,
						   COLUMN_GENRE, info->genre,
						   COLUMN_COMMENT,
						   info->comment,
						   COLUMN_EDITABLE, TRUE,
						   -1);
			}
			break;
		case CDDB_MATCH_NOMATCH:
			g_print("No match\n");
			break;
		}
	}
}
