#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "wav_cb.h"
#include "vorbis_cb.h"

GtkWidget *appbar;
GtkWidget *application;

static GnomeUIInfo file_menu_uiinfo[] = {
	GNOMEUIINFO_MENU_EXIT_ITEM(on_exit_activate, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo settings_menu_uiinfo[] = {
	GNOMEUIINFO_MENU_PREFERENCES_ITEM(on_preferences_activate, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo help_menu_uiinfo[] = {
	GNOMEUIINFO_MENU_ABOUT_ITEM(on_about_activate, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo menubar_uiinfo[] = {
	GNOMEUIINFO_MENU_FILE_TREE(file_menu_uiinfo),
	GNOMEUIINFO_MENU_SETTINGS_TREE(settings_menu_uiinfo),
	GNOMEUIINFO_MENU_HELP_TREE(help_menu_uiinfo),
	GNOMEUIINFO_END
};

GtkWidget *create_application(void)
{
	GtkWidget *toolbar;
	GtkWidget *tmp_toolbar_icon;
	GtkWidget *btn_scan;
	GtkWidget *btn_open;
	GtkWidget *btn_enc2wav;
	GtkWidget *btn_enc2mp3;
	GtkWidget *btn_enc2ogg;
	GtkWidget *btn_enc2ac3;
	GtkWidget *btn_quit;
	GtkTreeModel *tracks_list_model;
	GtkWidget *tracks_list;
	GtkCellRenderer *renderer;
	GtkWidget *scrolled_window;

	/* main window */
	application = gnome_app_new(PACKAGE, PACKAGE);
	gtk_window_set_default_size(GTK_WINDOW(application), 600, 400);
	g_signal_connect(G_OBJECT(application), "delete_event",
			 G_CALLBACK(on_exit_activate), NULL);


	/* menu */
	gnome_app_create_menus(GNOME_APP(application), menubar_uiinfo);

    /**************/
	/* track list */
    /**************/
	scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	gnome_app_set_contents(GNOME_APP(application), scrolled_window);

	/* create list store */
	tracks_list_model =
	    (GtkTreeModel *) gtk_list_store_new(NUM_COLUMNS,
						G_TYPE_BOOLEAN,
						G_TYPE_STRING,
						G_TYPE_STRING,
						G_TYPE_STRING,
						G_TYPE_STRING,
						G_TYPE_STRING,
						G_TYPE_STRING,
						G_TYPE_STRING,
						G_TYPE_STRING,
						G_TYPE_BOOLEAN);

	/* create tree view */
	tracks_list = gtk_tree_view_new_with_model(tracks_list_model);
	gtk_tree_selection_set_mode(gtk_tree_view_get_selection
				    (GTK_TREE_VIEW(tracks_list)),
				    GTK_SELECTION_MULTIPLE);


	/* rip column */
	renderer = gtk_cell_renderer_toggle_new();
	g_signal_connect(G_OBJECT(renderer), "toggled",
			 G_CALLBACK(on_cell_toggled_activate),
			 tracks_list_model);
	g_object_set_data(G_OBJECT(renderer), "column",
			  (gint *) COLUMN_SELECTED);
	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW
						    (tracks_list), -1,
						    "rip", renderer,
						    "active",
						    COLUMN_SELECTED, NULL);
	/* track column */
	renderer = gtk_cell_renderer_text_new();
	g_signal_connect(G_OBJECT(renderer), "edited",
			 G_CALLBACK(on_cell_edited_activate),
			 tracks_list_model);
	g_object_set_data(G_OBJECT(renderer), "column",
			  (gint *) COLUMN_TRACK);
	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW
						    (tracks_list), -1,
						    "track", renderer,
						    "text", COLUMN_TRACK,
						    "editable",
						    COLUMN_EDITABLE, NULL);

	/* time column */
	renderer = gtk_cell_renderer_text_new();
	g_signal_connect(G_OBJECT(renderer), "edited",
			 G_CALLBACK(on_cell_edited_activate),
			 tracks_list_model);
	g_object_set_data(G_OBJECT(renderer), "column",
			  (gint *) COLUMN_TIME);
	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW
						    (tracks_list), -1,
						    "time", renderer,
						    "text", COLUMN_TIME,
						    "editable",
						    COLUMN_EDITABLE, NULL);

	/* artist column */
	renderer = gtk_cell_renderer_text_new();
	g_signal_connect(G_OBJECT(renderer), "edited",
			 G_CALLBACK(on_cell_edited_activate),
			 tracks_list_model);
	g_object_set_data(G_OBJECT(renderer), "column",
			  (gint *) COLUMN_ARTIST);
	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW
						    (tracks_list), -1,
						    "artist", renderer,
						    "text", COLUMN_ARTIST,
						    "editable",
						    COLUMN_EDITABLE, NULL);

	/* title column */
	renderer = gtk_cell_renderer_text_new();
	g_signal_connect(G_OBJECT(renderer), "edited",
			 G_CALLBACK(on_cell_edited_activate),
			 tracks_list_model);
	g_object_set_data(G_OBJECT(renderer), "column",
			  (gint *) COLUMN_TITLE);
	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW
						    (tracks_list), -1,
						    "title", renderer,
						    "text", COLUMN_TITLE,
						    "editable",
						    COLUMN_EDITABLE, NULL);

	/* album column */
	renderer = gtk_cell_renderer_text_new();
	g_signal_connect(G_OBJECT(renderer), "edited",
			 G_CALLBACK(on_cell_edited_activate),
			 tracks_list_model);
	g_object_set_data(G_OBJECT(renderer), "column",
			  (gint *) COLUMN_ALBUM);
	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW
						    (tracks_list), -1,
						    "album", renderer,
						    "text", COLUMN_ALBUM,
						    "editable",
						    COLUMN_EDITABLE, NULL);

	/* year column */
	renderer = gtk_cell_renderer_text_new();
	g_signal_connect(G_OBJECT(renderer), "edited",
			 G_CALLBACK(on_cell_edited_activate),
			 tracks_list_model);
	g_object_set_data(G_OBJECT(renderer), "column",
			  (gint *) COLUMN_YEAR);
	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW
						    (tracks_list), -1,
						    "year", renderer,
						    "text", COLUMN_YEAR,
						    "editable",
						    COLUMN_EDITABLE, NULL);


	/* genre column */
	renderer = gtk_cell_renderer_text_new();
	g_signal_connect(G_OBJECT(renderer), "edited",
			 G_CALLBACK(on_cell_edited_activate),
			 tracks_list_model);
	g_object_set_data(G_OBJECT(renderer), "column",
			  (gint *) COLUMN_GENRE);
	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW
						    (tracks_list), -1,
						    "genre", renderer,
						    "text", COLUMN_GENRE,
						    "editable",
						    COLUMN_EDITABLE, NULL);

	/* comment column */
	renderer = gtk_cell_renderer_text_new();
	g_signal_connect(G_OBJECT(renderer), "edited",
			 G_CALLBACK(on_cell_edited_activate),
			 tracks_list_model);
	g_object_set_data(G_OBJECT(renderer), "column",
			  (gint *) COLUMN_COMMENT);
	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW
						    (tracks_list), -1,
						    "comment", renderer,
						    "text", COLUMN_COMMENT,
						    "editable",
						    COLUMN_EDITABLE, NULL);

	/* add list to main window */
	gtk_container_add(GTK_CONTAINER(scrolled_window), tracks_list);

    /***********/
	/* toolbar */
    /***********/
	toolbar = gtk_toolbar_new();
	gnome_app_add_toolbar(GNOME_APP(application), GTK_TOOLBAR(toolbar),
			      "toolbar", BONOBO_DOCK_ITEM_BEH_EXCLUSIVE,
			      BONOBO_DOCK_TOP, 1, 0, 0);


	/* scan cd-rom */
	tmp_toolbar_icon = gtk_image_new_from_stock(GTK_STOCK_CDROM,
						    GTK_ICON_SIZE_LARGE_TOOLBAR);
	btn_scan =
	    gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
				       GTK_TOOLBAR_CHILD_BUTTON, NULL,
				       "Scan", "Scan CD-ROM", NULL,
				       tmp_toolbar_icon,
				       GTK_SIGNAL_FUNC(on_scancd_activate),
				       tracks_list_model);

	/* CDDB */
	tmp_toolbar_icon =
	    gnome_pixmap_new_from_file_at_size(PACKAGE_PIXMAPS_DIR "cddb.png", 30, 30);
	btn_scan =
	    gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
				       GTK_TOOLBAR_CHILD_BUTTON, NULL,
				       "CDDB",
				       "Get information from internet",
				       NULL, tmp_toolbar_icon,
				       GTK_SIGNAL_FUNC(on_cddb_activate),
				       tracks_list_model);

	/* space */
	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));

	/* open files */
	tmp_toolbar_icon = gtk_image_new_from_stock(GTK_STOCK_OPEN,
						    GTK_ICON_SIZE_LARGE_TOOLBAR);
	btn_open =
	    gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
				       GTK_TOOLBAR_CHILD_BUTTON, NULL,
				       "Open", "Open files", NULL,
				       tmp_toolbar_icon, NULL, NULL);

	/* space */
	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));

	/* cd to wav */
	tmp_toolbar_icon = gnome_pixmap_new_from_file(PACKAGE_PIXMAPS_DIR "wav.png");
	btn_enc2wav =
	    gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
				       GTK_TOOLBAR_CHILD_BUTTON, NULL,
				       "wav", "rip to wav", NULL,
				       tmp_toolbar_icon,
				       GTK_SIGNAL_FUNC(on_towav_activate),
				       NULL);

	/* cd to mp3 */
	tmp_toolbar_icon = gnome_pixmap_new_from_file(PACKAGE_PIXMAPS_DIR "mp3.png");
	btn_enc2mp3 =
	    gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
				       GTK_TOOLBAR_CHILD_BUTTON, NULL,
				       "mp3", "rip to mp3", NULL,
				       tmp_toolbar_icon, NULL, NULL);

	/* cd to ogg */
	tmp_toolbar_icon = gnome_pixmap_new_from_file(PACKAGE_PIXMAPS_DIR "ogg.png");
	btn_enc2ogg =
	    gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
				       GTK_TOOLBAR_CHILD_BUTTON, NULL,
				       "ogg", "rip to ogg", NULL,
				       tmp_toolbar_icon,
				       GTK_SIGNAL_FUNC(on_toogg_activate),
				       tracks_list_model);

	/* cd to ac3 */
	tmp_toolbar_icon = gnome_pixmap_new_from_file(PACKAGE_PIXMAPS_DIR "ac3.png");
	btn_enc2ac3 =
	    gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
				       GTK_TOOLBAR_CHILD_BUTTON, NULL,
				       "ac3", "rip to ac3", NULL,
				       tmp_toolbar_icon, NULL, NULL);

	/* space */
	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));

	/* quit */
	tmp_toolbar_icon =
	    gtk_image_new_from_stock(GTK_STOCK_QUIT,
				     GTK_ICON_SIZE_LARGE_TOOLBAR);
	btn_quit =
	    gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
				       GTK_TOOLBAR_CHILD_BUTTON, NULL,
				       _("Quit"), _("Quit this program"), NULL,
				       tmp_toolbar_icon,
				       GTK_SIGNAL_FUNC(on_exit_activate),
				       NULL);


	/* statusbar */
	appbar = gnome_appbar_new(TRUE, TRUE, GNOME_PREFERENCES_NEVER);
	gnome_app_set_statusbar(GNOME_APP(application), appbar);

	/* hints */
	gnome_app_install_menu_hints(GNOME_APP(application),
				     menubar_uiinfo);

	/* show all widgets */
	gtk_widget_show_all(appbar);
	gtk_widget_show_all(toolbar);
	gtk_widget_show_all(scrolled_window);
	gtk_widget_show_all(tracks_list);
	return (application);
}
