// $Id: userdir.c,v 1.7 2000/02/21 00:49:04 siefca Exp $
#include "userdir.h"

static char *nonulhome = "";
static char variablebuf[MAXPATHLEN];
char *getvarbydotfile (char *variable, char *home_directory);
int preparemain(struct cfv *CFV);
int preparehome(struct cfv *CFV);
/******************************************************************************/

char *getvarbydotfile (char *variable, 
		       char *home_directory)
{
    char *p, *var, *retptr;
    char line[256];
    char buf[MAXPATHLEN];
    FILE *cfile;

    *variablebuf = '\0';
    retptr = NULL;
    snprintf(buf,  MAXPATHLEN - 1, "%s/%s", home_directory, DOTFILENAME);
    /* tu niech bada czy jest prawo zapisu dla innych */
    cfile = fopen (buf, "r");
    if (cfile == NULL) return (NULL);
    line[255] = '\0';
    while (fgets(line, 255, cfile))
	{
	if ((p=strchr(line, '#'))) *p = '\0';
	if ((p=strchr(line, '\n'))) *p = '\0';
	if (*line == ' ' || *line == '\t')
	    {
	    p = line;
	    do p++;
	    while (*p != '\0' && (*p == ' ' || *p == '\t'));
	    if (p == '\0') continue; /* syntax error */
	    var = p;
	    }
	else var = line;
	if (   (p=strchr(line, ' ')) 
	    || (p=strchr(line, '\t')) 
	    || (p=strchr(line, '=')) )
	    {
	    *p = '\0';
	    do p++;
	    while (   *p != '\0' 
	           && (   *p == ' ' 
		       || *p == '\t' 
		       || *p == '=')
		  );
	    if (p == '\0') continue; /* syntax error */
	    }
	else continue; /* syntax error */
	
	if (!strcmp(var, variable)) /* have got it! */
	    {
	    strncpy (variablebuf, p, MAXPATHLEN - 1);
	    retptr = variablebuf;
	    break;
	    }
	}
    fclose (cfile);
#ifdef MAP_FILE_LOOKUP_TO_ENV
    snprintf(buf, MAXPATHLEN, "%s=%s", variable, retptr);
    (void) putenv (buf);
#endif
    return (retptr);
}

/******************************************************************************/

int getusercfv(char *variable,
	       char *home_dir,
	       char *user_dir,
	       size_t stringsize)
{
    struct stat st;
    char *d;

    bzero(user_dir, stringsize);
    if (variable == NULL || *variable == '\0')
	return (-1);
    if (home_dir == NULL)
	home_dir = nonulhome;
    else
	{ /* remove last / in home_dir */
	if ((d = strrchr(home_dir, '/')) && *(d+1) == '\0')
	    *d = '\0';
	}
    d = getenv(variable);    
    if (d == NULL || *d == '\0') /* don't have variable */
	{
	d = getvarbydotfile (variable, home_dir);
	if (d == NULL || *d == '\0') /* don't have variable */
	    {
	    errno = ENOENT;
	    return (-1);
	    }
	}
    if (*d == '/') {
	strncpy(user_dir, d, stringsize - 1);
    } else {
	snprintf(user_dir, stringsize - 1, "%s/%s", home_dir, d);
    }
    if (stat(user_dir, &st) != -1 && S_ISDIR(st.st_mode)) {
	return (0);
    }
    errno = ENOENT;
    return (-1);
}

/******************************************************************************/

int detectcfv(struct cfv *CFV)
{
    if (CFV->subname == NULL)
	CFV->subname = nonulhome;
    if (CFV->home_dir == NULL)
	CFV->home_dir = getenv("HOME");
    if (CFV->home_dir == NULL)
	CFV->home_dir = CFV->home_scd;
    if (CFV->home_dir == NULL)
	{
	errno = ENOENT;
	return (-1);
	}
    return 0;
}

/******************************************************************************/

int preparemain(struct cfv *CFV)
{
    char *p;
    struct stat st;
    if ((p = strrchr(CFV->result, '/')) && *(p + 1) == '\0')
	p = '\0';
    snprintf((rindex(CFV->result, '\0')), MAXPATHLEN - 1, "/%s",
	     CFV->subname);
    if (stat(CFV->result, &st) != -1
	&& CFV->mode ?
	S_ISDIR(st.st_mode):
	S_ISREG(st.st_mode))
	return (1);
    return 0;
}

/******************************************************************************/

int preparehome(struct cfv *CFV)
{
    struct stat st;

    if (CFV->home_dir == NULL || *(CFV->home_dir) == '\0')
	return (-1);

    snprintf(CFV->result, MAXPATHLEN - 1, "%s/%s%s%s",
	     CFV->home_dir,
	     CFV->prefix ? CFV->prefix : "",
	     CFV->subname,
	     CFV->suffix ? CFV->suffix : "");
	     
    if (stat(CFV->result, &st) != -1
	&& CFV->mode ?
	S_ISDIR(st.st_mode) :
	S_ISREG(st.st_mode))	/* have file or dir */
	return (2);

    return (-1);
}

/******************************************************************************/

int usercfv(struct cfv *CFV)
{
    int gr;

    CFV->athome = 0;
    gr = detectcfv(CFV);
    if (gr)
	return gr;

    /* get environment variable */
    gr = getusercfv(CFV->variable,
		    CFV->home_dir,
		    CFV->result,
		    MAXPATHLEN);
		    
    if (gr != -1) {		/* have main directory + read permission */
	gr = preparemain(CFV);
	if (gr && !access(CFV->result, CFV->mode ? X_OK : R_OK))
	    return 0;
	}
    /* don't have main directory or a proper variable set */
    gr = preparehome(CFV);
    if (gr == -1) return (-1);
    if (access(CFV->result, CFV->mode ? X_OK : R_OK))
	return (-1);
    CFV->athome = 1;
    return 1;
}

/******************************************************************************/

/* If directory or file doesn't exist returns the best one wich may be created */
int notnullusercfv(struct cfv *CFV)
{
    char buf[MAXPATHLEN];
    int gr;
    char *p;

    if ((gr = usercfv(CFV)) == -1) {
	gr = detectcfv(CFV);
	if (gr)
	    return gr;		/* remember, it works for errors! */
	/* get the environment variable */
	gr = getusercfv(CFV->variable,
			CFV->home_dir,
			CFV->result,
			MAXPATHLEN);
			
	(void) preparemain(CFV);
	strncpy(buf, CFV->result, MAXPATHLEN-1);
	p = strrchr(buf, '/');
	if (p) *p = '\0';
	if (!access(buf, W_OK))
	    return 0;

	/* don't have main directory or a proper variable set */
	/* or not enough permissions			      */
	(void) preparehome(CFV);
	strncpy(buf, CFV->result, MAXPATHLEN-1);
	p = strrchr(buf, '/');
	if (p) *p = '\0';
	if (access(buf, W_OK))
	    return (-1);
	CFV->athome = 1;
	return 1;
    }
    return (gr);
}

/******************************************************************************/

int chdircfv(struct cfv *CFV)
{
    if ((usercfv(CFV)) == -1)
	return (-1);
    return ( chdir(CFV->result) );
}

/******************************************************************************/

int forcechdircfv(struct cfv *CFV, mode_t mode)
{
    int ret;
    
    ret = chdircfv(CFV);
    if (ret != -1) 
	return 0;
	
    ret = notnullusercfv(CFV);
    
    if (ret != -1) /* have the best location */
	{
	ret = mkdir(CFV->result, mode);
	if (ret == -1) return ret;
	ret = chdir(CFV->result);
	if (ret == -1) return ret;
	return 1;
	}
    return (-1);
}

/******************************************************************************/

int checkfilecfv (struct cfv *CFV, mode_t mode)
{
  int fd;

  if ((usercfv(CFV)) != -1)
      return 1; /* already present! */

  if ((notnullusercfv(CFV)) == -1)
      return (-1);
      
  fd = open (CFV->result, O_CREAT|O_WRONLY|O_APPEND, mode);
  if (fd == -1)
      return (-1);
  close (fd);
  return (0);
}

/******************************************************************************/

int checkdircfv (struct cfv *CFV, mode_t mode)
{
  if ((usercfv(CFV)) != -1)
      return 1; /* already present! */
  if ((notnullusercfv(CFV)) == -1)
      return (-1);
  return ( mkdir(CFV->result, mode) );
}

/******************************************************************************/

int checkoutcfv (struct cfv *CFV, mode_t mode)
{
  if (CFV->mode == M_DIRECTORY)
      return ( checkdircfv (CFV, mode) );
  return ( checkfilecfv (CFV, mode) );
}

/******************************************************************************/

FILE *maynullfopencfv(struct cfv * CFV, const char *mode)
{
    FILE *cfvfile = NULL;

    if ((usercfv(CFV)) == -1)
    	{
	errno = ENOENT;
	return (NULL);
	}
    cfvfile = fopen(CFV->result, mode);
    return (cfvfile);
}
/******************************************************************************/

FILE *notnullfopencfv(struct cfv * CFV, const char *mode)
{
    FILE *cfvfile = NULL;
    
    if ((notnullusercfv(CFV)) == -1)
	return (NULL);

    cfvfile = fopen(CFV->result, mode);
    return (cfvfile);
}

/******************************************************************************/

FILE *fopencfv(struct cfv * CFV, const char *mode)
{
    FILE *cfvfile = NULL;

    if ((strpbrk(mode, "wa")) != NULL)	/* create in best location */
	cfvfile = notnullfopencfv(CFV, mode);
    else
	cfvfile = maynullfopencfv(CFV, mode);

    return (cfvfile);
}

/******************************************************************************/

DIR *opendircfv (struct cfv * CFV)
{
    DIR *cfvdir = NULL;

    if ((usercfv(CFV)) == -1)
	{
	errno = ENOENT;
	return (NULL);
	}
    cfvdir = opendir(CFV->result);
    return (cfvdir);
}

/******************************************************************************/

void *opencfv(struct cfv * CFV, const char *mode)
{
    if (CFV->mode == M_DIRECTORY ||
        mode == NULL || 
        *mode == '\0' || 
	(strchr(mode,'d')) != NULL)
	return ( (void*) opendircfv (CFV) );

    return ( (void*) fopencfv (CFV, mode) );
}

/******************************************************************************/
