// $Id: userdir.h,v 1.9 2000/02/21 00:49:04 siefca Exp $
#ifndef USERDIR__H
#define USERDIR__H

#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <dirent.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <errno.h>

#define	M_REGULAR_FILE	0	/* DO NOT SWITCH IT! */
#define	M_DIRECTORY	1
#define	DOTFILENAME	".userdir"

/* 
 * Should I map lookups from .userdir to 
 * environment variable ???
 *
 */

#define	MAP_FILE_LOOKUP_TO_ENV

struct cfv {
	char result[MAXPATHLEN];	/* our result			*/
	char *variable;	/* name of an environment variable		*/
	char *home_dir;	/* home directory or NULL for autodetect	*/
	char *home_scd; /* directory if home_dir==NULL and no result	*/
	char *subname;	/* core name of a file/directory		*/
    	char *prefix;	/* prefix when using directly home_dir		*/
	char *suffix;	/* suffix when using directly home_dir		*/
	int mode;	/* expected: M_REGULAR_FILE or M_DIRECTORY	*/
	unsigned athome:1; /* are we placed inside of home directory?	*/
	};

/****************** CORE FUNCTIONS  ******************/

/* reads environment variable. if the path isn't absolute will add $HOME/
 * at the beginning
 * return: 0 - directory exists
 *         -1 - directory doesn't exist
 */
int getusercfv (char *variable, 
		    char *home_dir, 
		    char *user_dir,
		    size_t stringsize);
/* reads the $HOME variable */
int detectcfv(struct cfv *CFV);

/* looks for the config/data file/dir.
 * result: -1 - error - cannot find file/dir 
 *         0 - ok
 *         CFV.result set
 */
int usercfv (struct cfv *CFV);

/* looks for the config/data file/dir.
 * result: -1 - error - cannot read $HOME, not enough create permissions
 *          0 - ok - if the CFV.result exists, read it. If not - select
 *                   the best location for creation.
 *              CFV.result set
 */              
int notnullusercfv (struct cfv *CFV);

/************** GENERAL CHECKOUT FUNCTION **************/

/* It tries to open file or directory, and if it doesn't
 * exists it tries to create it in the best location. 
 */
int checkoutcfv (struct cfv *CFV, mode_t mode);

/************* OPENING THE DIRECTORY/FILE **************/

/* opens file or directory using CFV structure
 *  if (CFV.mode is set to M_DIRECTORY 
 *  or mode is NULL 
 *  or mode is set to "" 
 *  or mode contains 'd' character) -> opendircfv is invoked
 *  else fopencfv is invoked
 * result: pointer to DIR or FILE handle (conversion need)
 */
void *opencfv(struct cfv * CFV, const char *mode);

/****************** CREATING THE FILE *********************/
/*   if it doesn't exists				  */
int checkfilecfv (struct cfv *CFV, mode_t mode);

/*  NOTE:					*/
/*						*/
/* Some maynull-like functions specified bellow	*/
/* can set errno variable to ENOENT if the file	*/
/* or directory doesn't exists.			*/

/****************** OPENING THE FILE ******************/

/* opens file using CFV and the given mode
 * after getting name fopen is invoked.
 * if the given file is considered to be opened for
 * write it is created.
 * result: [see fopen(3)]
 */
FILE *fopencfv (struct cfv *CFV, const char *mode);

/* opens file using CFV and the given mode
 * just after getting filename, fopen is invoked
 * if specified file doesn't exists error is returned
 * result: [see fopen(3)] or NULL if cannot find the file
 */
FILE *maynullfopencfv (struct cfv *CFV, const char *mode);

/* opens file using CFV and the given mode
 * after getting name fopen is invoked
 * if the specified file doesn't exists it's created
 * in the best location
 * result: [see fopen(3)]
 */
FILE *notnullfopencfv (struct cfv *CFV, const char *mode);

/****************** OPENING THE DIRECTORY ****************/

/* opens directory using CFV
 * after getting name opendir is invoked
 * result: [see opendir(3)] or NULL if the
 *         directory doesn't exists
 */
DIR *opendircfv (struct cfv * CFV);

/****************** CREATING THE DIRECTORY ****************/
/*   if it doesn't exists				  */

int checkdircfv (struct cfv *CFV, mode_t mode);

/****************** CHANGING THE DIRECTORY ****************/

/* tries to change current working directory using CFV
 * after getting name chdir is invoked
 * result: [see chdir(2)] or -1 if it doesn't exists
 */
int chdircfv (struct cfv *CFV);

/* tries to change current working directory using CFV
 * after getting name chdir is invoked
 * if the specified directory doesn't exists it's created
 * result: [see chdir(2)] 
 */
int forcechdircfv (struct cfv *CFV, mode_t mode);

#endif
