#include "wav_cb.h"
#include "cdrom_handler.h"
#include "settings.h"
#include "error.h"
#include "progress.h"
#include <pthread.h>
#include <gnome.h>

static gint update(gpointer data);

/********************************************************************/
void on_towav_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	state.progress_state = PROGRESS_WAV;
	on_towav_activate2(menuitem, user_data);
	state.scan = FALSE;
}

/********************************************************************/
static void save_path(gint i, gchar * path)
{
	struct track_info *track = g_slist_nth_data(tracks_list, i);
	gchar *tmp = state.general_name_format;
	gchar *file = "";

	while (*tmp != '\0') {
		if (*tmp == '%') {
			tmp++;
			switch (*tmp) {
			case 'p':
				file =
				    g_strdup_printf("%s%s", file,
						    track->artist);
				break;
			case 't':
				file =
				    g_strdup_printf("%s%s", file,
						    track->title);
				break;
			case 'a':
				file =
				    g_strdup_printf("%s%s", file,
						    track->album);
				break;
			case 'y':
				file =
				    g_strdup_printf("%s%s", file,
						    track->year);
				break;
			case 'n':
				file =
				    g_strdup_printf("%s%d", file,
						    track->track);
				break;
			case 'c':
				file =
				    g_strdup_printf("%s%s", file,
						    track->comment);
				break;
			}
		} else {
			file =
			    g_strdup_printf("%s%s", file,
					    g_strndup(tmp, 1));
		}
		tmp++;
	}
	track->openfile = g_strdup_printf("%s%s.wav", path, file);
	track->savefile = g_strdup_printf("%s%s.wav", path, file);
	g_free(file);
}

/********************************************************************/
static void on_fileselector_ok(GtkWidget * widget, gpointer data)
{
	gint i;
	struct track_info *track = NULL;

	/* save path */
	for (i = 0; i < g_slist_length(tracks_list); i++)
		save_path(i, (gchar *)
			  gtk_file_selection_get_filename
			  (GTK_FILE_SELECTION(data)));

	/* let's rip! */
	state.totalsize = 0;
	state.ripsize = 0;
	for (i = 0; i < g_slist_length(tracks_list); i++) {
		track = g_slist_nth_data(tracks_list, i);
		if (track->selected == TRUE) {
			state.totalsize += track->wavsize;
			pthread_create(&state.thread, NULL,
				       cdrom_get_track,
				       (void *) GINT_TO_POINTER(i));
			pthread_detach(state.thread);
			gtk_timeout_add(500, update, NULL);
		}
	}
}

/********************************************************************/
int on_towav_activate2(GtkMenuItem * menuitem, gpointer user_data)
{
	GtkWidget *file_window;

	/* check if cd was scan */
	if ((state.scan == FALSE) && (state.cd == TRUE))
		return (GR_ERROR);

	/* check if list is empty */
	if (g_slist_length(tracks_list) == 0)
		return (GR_ERROR);

	/* choose directory to save files */
	file_window =
	    gtk_file_selection_new("Choose directory to save files...");

	g_signal_connect(G_OBJECT
			 (GTK_FILE_SELECTION(file_window)->ok_button),
			 "clicked", G_CALLBACK(on_fileselector_ok),
			 file_window);
	g_signal_connect_swapped(G_OBJECT
				 (GTK_FILE_SELECTION(file_window)->
				  ok_button), "clicked",
				 G_CALLBACK(gtk_widget_destroy),
				 file_window);
	g_signal_connect_swapped(G_OBJECT
				 (GTK_FILE_SELECTION(file_window)->
				  cancel_button), "clicked",
				 G_CALLBACK(gtk_widget_destroy),
				 file_window);



	gtk_widget_show(file_window);

	return (GR_SUCCESS);
}

/********************************************************************/
static gint update(gpointer data)
{
	if ((state.progress_state > PROGRESS_ABORT) && (progress.window == NULL))
		gr_progress_new("");

	gr_progress_update(state.progress, state.progress2,
			   state.progress_text);

	if ((state.totalsize != 0) && (state.totalsize == state.ripsize)
	    && (state.progress_state == PROGRESS_WAV))
		state.progress_state = PROGRESS_ABORT;

	if (state.progress_state == PROGRESS_ABORT) {
		gr_progress_destroy(&progress);
		return (FALSE);
	}

	return (TRUE);
}
